"""
Compatibility shims for getting stuff from test.support across
Python versions (for compatibility with Python 3.9 and earlier).

>>> os_helper = try_import('os_helper') or from_test_support('temp_dir')
>>> os_helper.temp_dir
<function temp_dir at ...>
"""

from __future__ import annotations

import importlib
import types

from jaraco.collections import Projection
from jaraco.context import suppress


def from_test_support(*names: str) -> types.SimpleNamespace:
    """
    Return a SimpleNamespace of names from test.support.

    >>> support = from_test_support('swap_item')
    >>> support.swap_item
    <function swap_item at ...>
    """
    import test.support

    return types.SimpleNamespace(**Projection(names, vars(test.support)))


@suppress(ImportError)
def try_import(name: str) -> types.ModuleType:
    """
    Attempt to import a submodule of test.support; return None if missing.
    """
    return importlib.import_module(f'test.support.{name}')



from __future__ import annotations

import urllib.request

import jaraco.functools
import pytest
from jaraco.context import ExceptionTrap


@jaraco.functools.once
def has_internet() -> bool:
    """
    Is this host able to reach the Internet?

    Return True if the internet appears reachable and False
    otherwise.
    """
    with ExceptionTrap() as trap:  # type: ignore[no-untyped-call] # jaraco/jaraco.context#15
        urllib.request.urlopen('http://pypi.org')
    return not trap


def check_internet() -> None:
    """
    (pytest) Skip if internet is unavailable.
    """
    if not has_internet():
        pytest.skip('Internet connectivity unavailable')


@pytest.fixture
def needs_internet() -> None:
    """
    Pytest fixture signaling that internet is required.
    """
    check_internet()


def pytest_configure(config: pytest.Config) -> None:
    """
    Register the 'network' marker.
    """
    config.addinivalue_line(
        "markers", "network: the test requires network connectivity"
    )


def pytest_runtest_setup(item: pytest.Item) -> None:
    """
    For any tests marked with 'network', install fixture.
    """
    for marker in item.iter_markers(name='network'):
        item.fixturenames.extend({'needs_internet'} - set(item.fixturenames))  # type: ignore[attr-defined]



from __future__ import annotations

import shutil
import subprocess

import pytest


def _resolve_git() -> str | None:
    """Locate a functional git executable if available."""
    try:
        subprocess.run(
            ['git', '--version'],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
    except Exception:
        return None
    return shutil.which('git')


@pytest.fixture
def ensure_git() -> None:
    """
    Require a working git executable for the test.

    >>> getfixture('ensure_git')
    """
    _resolve_git() or pytest.skip("'git' command unavailable")



from __future__ import annotations

import sys


def property_error(name: str) -> str:
    """
    Generate a regular expression for capturing the expected
    error messages that can result from attempting to set
    on a property without a setter.

    >>> class Foo:
    ...     @property
    ...     def bar(self):
    ...         return 'bar'
    >>> import pytest
    >>> with pytest.raises(AttributeError, match=property_error('Foo.bar')):
    ...     Foo().bar = 'anything'
    """
    class_, property_ = name.split('.')
    return (
        "can't set attribute"
        if sys.version_info < (3, 10)
        else f"can't set attribute {property_!r}"
        if sys.version_info < (3, 11)
        else f"property {property_!r} of {class_!r} object has no setter"
    )
